#pragma once

struct lkt_state;

/* Store functions with a name, so that all the lib will have acces to it
   with a name that can be generated or red from a config file. */
struct module_reg {
    const char *name;
    void (*func)(void);
};

extern struct module_reg *reg;

#define REG_BEGIN(reg)          struct module_reg reg[] = {
#define REG_ADD(__f)            { .name = #__f,  .func = (void(*)(void)) __f    },
#define REG_ADD_NAMED(__n, __f) { .name = __n,   .func = (void(*)(void)) __f    },
#define REG_END()               { .name = NULL,  .func = NULL                   } };

/* If handle is NULL, file is unused and the reg is red. Otherwise,
   we use dlfcn to search for the symbol which is returned. If *handle
   is NULL, it will be created by opening the file. */
void *reg_pick(const char *file, void **handle, const char *symbol);

/* Set the ref for this .a / .so file */
void reg_set(struct module_reg *);

/* Load a module by its name. Also initialize the mod pointer. No check on data
   type is done, it is up to the user to check if the structure passed as a
   `void*` is the right structure. */
int load_module_by_name(struct lkt_state *srv, const char *name, void *mod);
