#pragma once

/* The window is a module */

#include <common/common.h>
#include <stdbool.h>
#include <sqlite3.h>

typedef void *lkt_win_element_t;    /* The real window struct, an opaque type   */
struct lkt_state;                   /* Forward definition of the server         */

/* The window with the selected correct functions, this is done because we may
   need an SDL2, a X11 or a QT window, and may want to select the one to use at
   run time. */
struct lkt_win {
    lkt_win_element_t window;           /* the opaque window type that depend on the module */
    void *handle;                       /* the libdl handle                                 */
    struct lkt_state *srv;              /* The server, must be set before the new call      */

    /* Create and free the window */
    bool (*new)(struct lkt_win *win);   /* Create a window or only the mpv context  */
    void (*close)(struct lkt_win *win); /* Close the mpv context, not the window    */
    void (*free)(struct lkt_win *win);  /* Entirelly liberate all the resources     */

    /* Playback control */
    bool (*toggle_pause)(struct lkt_win *win);
    bool (*load_file)(struct lkt_win *win, const char *filepath);
    bool (*set_volume)(struct lkt_win *win, int vol);

    /* Get playback properties */
    bool (*get_duration)(struct lkt_win *win, int *dur_sec);
    bool (*get_elapsed)(struct lkt_win *win, int *elapsed_sec);

    /* Handle loop event */
    bool (*handle_events)(struct lkt_win *win, volatile sqlite3 *db,
                          mpd_idle_flag *mpd_idle_events);
};
