#define _POSIX_C_SOURCE 200809L

#include <common/common.h>
#include <lektor/reg.h>
#include <lektor/database.h>
#include <lektor/config.h>
#include <lektor/net.h>
#include <dlfcn.h>
#include <stdio.h>
#include <strings.h>
#include <stddef.h>

struct module_reg *reg;

void *
reg_pick(const char *file, void **handle, const char *symbol)
{
    int i;
    if (!symbol)
        return NULL;

    /* Use the register instead of dlfcn? */
    if (!file || file[0] == '\0' || STR_MATCH(file, "STATIC") || handle == NULL)
        goto use_reg;

    /* Use dlsym */
    LOG_INFO("REG", "Using dlfcn to find %s", symbol);
    char *error;
    *handle = dlopen(file, RTLD_NOW);

    if (NULL == *handle) {
        LOG_ERROR("REG", "libdl error in dlopen on file '%s': %s",
                  file, dlerror());
        return NULL;
    }

    dlerror();
    void *sym = dlsym(*handle, symbol);

    if ((error = dlerror()) != NULL) {
        LOG_ERROR("REG", "libdl error in dlsym on file '%s': %s\n",
                  file, error);
        return NULL;
    }

    return sym;

    /* Read the reg */
use_reg:
    LOG_INFO("REG", "Using the reg structure to find %s", symbol);
    for (i = 0; reg[i].name && reg[i].func; ++i)
        if (STR_MATCH(reg[i].name, symbol))
            return *(void **) &reg[i].func;

    return NULL;
}

void
reg_set(struct module_reg *_reg)
{
    reg = _reg;
}

int
load_so(const char *const mod_path, const char *const mod_init, void *mod,
        struct lkt_state *srv)
{
    /* An init function takes two arguments: the module pointer and a void*
       which is the handler for the .so file. It the role of the module to
       store the handle pointer.
       Uppon successfull completion, the function shall return 0, and return
       a non zero value if something bad happened. */
    int (*module_set_function)(void *const, struct lkt_state *, void *const);
    void *handle = NULL;
    *(void **) (&module_set_function) = reg_pick(mod_path, &handle, mod_init);

    if (module_set_function)
        return module_set_function(mod, srv, handle);
    else {
        LOG_ERROR("LOAD", "Failed to find init symbol %s "
                  "in file %s", mod_init, mod_path);
        return 1;
    }
}

int
load_module_by_name(struct lkt_state *srv, const char *name, void *mod)
{
    char mod_path[PATH_MAX], mod_load[INI_MAX_LINE_LEN];

    /* When, don't mind if its not here */
    if (!database_config_get_text(srv->db, name, "path", mod_path, PATH_MAX)) {
        LOG_WARN("CONFIG", "No setting 'path' in section '%s'", name);
        mod_path[0] = '\0';
    }

    if (!database_config_get_text(srv->db, name, "load_function",
                                  mod_load, INI_MAX_LINE_LEN)) {
        LOG_ERROR("CONFIG", "Module named %s is incomplete or "
                  "is not defined in config file", name);
        return 1;
    }

    return load_so(mod_path, mod_load, mod, srv);
}

